import os
import unittest
import shutil
from commons.core.utils.FileUtils import FileUtils
from commons.core.sql.DbFactory import DbFactory

class Test_F_ProfilesSearchTEClassifierStep1(unittest.TestCase):

    HOST_NAME = "compute-2-46.local"
    DIR2CLEAN = "dir2clean"
    
    def setUp(self):
        os.mkdir(self.DIR2CLEAN)
        os.chdir(self.DIR2CLEAN)
        self._projectName = "test_profiles"
        self._configFileName = "TEdenovo.cfg"
        #TODO: data must be in REPET_DATA
        self._consensusFileName = "ConsensusTestFile_nt.fsa"
        self._consensusFilePath = "%s/commons/pyRepetUnit/align/transformAACoordIntoNtCoord/tests/datas/%s" % (os.environ.get("REPET_PATH"), self._consensusFileName)
        os.symlink(self._consensusFilePath, self._consensusFileName)
        self._bankName = "hmmbank_test"
        self._bankPath = "%s/commons/pyRepetUnit/hmmer/profilsSearchInTEClassifier/tests/datas/%s" % (os.environ.get("REPET_PATH"), self._bankName)
        os.symlink(self._bankPath, self._bankName)
        self._writeConfigFile()
        
    def tearDown(self):
        os.chdir("..")
        shutil.rmtree(self.DIR2CLEAN)
       
    def test_TEClassifier_Step1_profiles(self): 
        cmd = "TEclassifier.py"
        cmd += " -s 1"
        cmd += " -p %s" % self._projectName
        cmd += " -c %s" % self._configFileName
        cmd += " -i %s" % self._consensusFileName
        os.system(cmd) 
        db = DbFactory.createInstance()
        db.dropTable("%s_seq" % self._projectName)
        tableName = "%s_Profiles_path" % self._projectName
        self.assertTrue(db.doesTableExist(tableName))
        self._writeExpHmmer2("expPathFileHmmer2.path")
        self.assertTrue(FileUtils.are2FilesIdentical("expPathFileHmmer2.path", "HmmProfiles/%s_profiles_%s.align.clean_match.path" % (self._projectName, self._bankName)))
        db.dropTable(tableName)
        db.close()
        
    def test_TEClassifierStep1_New_Hmmer3(self): 
        cmd = "TEclassifier.py"
        cmd += " -s 1"
        cmd += " -p %s" % self._projectName
        cmd += " -c %s" % self._configFileName
        cmd += " -i %s" % self._consensusFileName
        cmd += " -n"
        os.system(cmd) 
        db = DbFactory.createInstance()
        db.dropTable("%s_seq" % self._projectName)
        tableName = "%s_Profiles_path" % self._projectName
        self.assertTrue(db.doesTableExist(tableName))
        self._writeExpHmmer3("expPathFileHmmer3.path")
        self.assertTrue(FileUtils.are2FilesIdentical("expPathFileHmmer3.path", "HmmProfiles/%s_profiles_%s.align.clean_match.path" % (self._projectName, self._bankName)))
        db.dropTable(tableName)
        db.close()
        
    def _writeConfigFile(self):
        f = open(self._configFileName, "w")
        f.write("[repet_env]\n")
        f.write("repet_user: %s\n" % os.environ.get("REPET_USER"))
        f.write("repet_host: %s\n" % os.environ.get("REPET_HOST"))
        f.write("repet_pw: %s\n" % os.environ.get("REPET_PW"))
        f.write("repet_db: %s\n" % os.environ.get("REPET_DB"))
        f.write("repet_port: %s\n" % os.environ.get("REPET_PORT"))
        f.write("\n")
        f.write("[project]\n")
        f.write("project_name: %s\n" % self._projectName)
        f.write("project_dir: %s\n" % os.getcwd())
        f.write("\n")
        f.write("[detect_features]\n")
        f.write("tmpDir: \n")
        f.write("clean: yes\n")
        f.write("te_hmmer: yes\n")
        f.write("te_hmmer_evalue: 10\n")
        f.write("te_hmm_profiles: %s\n" % self._bankName)
        if self.HOST_NAME == os.getenv("HOSTNAME"):
            f.write("resources: test=TRUE\n")
        else:
            f.write("resources: \n") 
        f.write("term_rep: no\n")
        f.write("polyA: no\n")
        f.write("tand_rep: no\n")
        f.write("orf: no\n")
        f.write("TE_BLRtx: no\n")
        f.write("TE_BLRx: no\n")
        f.write("HG_BLRn: no\n")
        f.write("blast: ncbi\n")
        f.write("TE_BLRn: no\n")
        f.write("rDNA_BLRn: no\n")
        f.write("tRNA_scan: no\n")
        f.close()
        
    def _writeExpHmmer2(self, fileName):
        f = open(fileName, "w")
        f.write("1\tblumeria_Grouper_4152_12:NoCat\t337\t381\tPeptidase_S29\t1\t15\t0.0034\t4\t0\n")
        f.write("2\tblumeria_Grouper_4152_12:NoCat\t62\t79\tToxin_18\t50\t55\t0.014\t4\t0\n")
        f.write("3\tblumeria_Grouper_4152_12:NoCat\t428\t478\tDUF1602\t23\t39\t0.0013\t5\t0\n")
        f.write("4\tblumeria_Grouper_4152_12:NoCat\t382\t407\tV-ATPase_G\t2\t10\t0.017\t2\t0\n")
        f.write("5\tblumeria_Grouper_4152_12:NoCat\t136\t228\tXhoI\t202\t172\t4.3e-05\t9\t0\n")
        f.write("6\tblumeria_Grouper_4152_12:NoCat\t479\t497\tDUF1798\t40\t35\t0.011\t1\t0\n")
        f.write("7\tblumeria_Grouper_4152_12:NoCat\t294\t365\tDUF881\t237\t214\t0.0052\t4\t0\n")
        f.write("8\tblumeria_Grouper_4152_12:NoCat\t84\t131\tLBP_BPI_CETP\t209\t191\t0.012\t1\t0\n")
        f.write("9\tblumeria_Grouper_4152_12:NoCat\t230\t247\tSOCS_box\t6\t1\t0.031\t2\t0\n")
        f.write("10\tblumeria_Grouper_590_20:NoCat\t271\t324\tDUF234\t5\t22\t0.0047\t3\t0\n")
        f.write("11\tblumeria_Grouper_590_20:NoCat\t331\t357\tDUF1414\t1\t9\t0.02\t2\t0\n")
        f.write("12\tblumeria_Grouper_590_20:NoCat\t90\t113\tCPW_WPC\t1\t9\t0.024\t1\t0\n")
        f.write("13\tblumeria_Grouper_590_20:NoCat\t119\t148\tDUF46\t182\t173\t0.00034\t6\t0\n")
        f.close()

    def _writeExpHmmer3(self, fileName):
        f = open(fileName, "w")
        f.write("1\tblumeria_Grouper_4152_12:NoCat\t232\t240\tTNV_CP\t187\t189\t0.00087\t1\t0\n")
        f.write("2\tblumeria_Grouper_4152_12:NoCat\t145\t231\tXhoI\t199\t171\t0.00024\t9\t0\n")
        f.write("3\tblumeria_Grouper_4152_12:NoCat\t438\t497\tDUF1798\t54\t35\t0.00053\t9\t0\n")
        f.write("4\tblumeria_Grouper_590_20:NoCat\t271\t324\tDUF234\t5\t22\t0.00029\t10\t0\n")
        f.close()

if __name__ == "__main__":                 
    unittest.main()
